/*
 *  Alex Haggblade (ahaggbla), Phyllis Douglas (pdouglas)
 *
 *****************************************************************************/
#ifdef __APPLE__
#include "GLUT/glut.h"
#include <OPENGL/gl.h>
#endif
#ifdef __unix__
#include <GL/glut.h>
#endif

#include <stdlib.h>
#include <stdio.h>

#include "GLSL_helper.h"
#include "glm/glm.hpp"
#include "glm/gtc/matrix_transform.hpp" //perspective, trans etc
#include "glm/gtc/type_ptr.hpp" //value_ptr

#include "RenderingHelper.h"
#include "objects/Bird.h"
#include "objects/Baseball.h"
#include "State.h"
#include "MainMenu.h"
#include "Game.h"
#include "objects/Textures.h"

#include "FinalProject.h"

#define TIMER 30 // ~33 fps
#define INIT_HEIGHT 100
#define MOUSE_SCALE 5

using namespace std;
using namespace glm;

//flag and ID to toggle on and off the shader
int ShadeProg, CloudShadeProg;

//Handles to the shader data
GLint h_aPosition;
GLint h_uAmbColor, h_uDiffColor, h_uSpecColor, h_uShine;
GLint h_uLColor;
GLint h_aNormal;
GLint h_uModelMatrix, h_uTpModelMatrix;
GLint h_uViewMatrix;
GLint h_uProjMatrix;

static float g_width, g_height;
float g_trans = -5.5;
float g_Mtrans = 0;
float g_angle = 0;
static float g_scale = 1;
GLuint g_textureid;

State* currState;

int getShade()
{
    return ShadeProg;
}

int getCloudShade()
{
    return CloudShadeProg;
}

float getWidth()
{
    return g_width;
}

float getHeight()
{
    return g_height;
}

float getTrans()
{
    return g_trans;
}

float getAngle()
{
    return g_angle;
}

/* projection matrix */
void SetProjectionMatrix()
{
    glm::mat4 Projection = glm::perspective(80.0f, (float) g_width / g_height, 0.1f, 100.f);
    safe_glUniformMatrix4fv(h_uProjMatrix, glm::value_ptr(Projection));
}

/* camera controls - do not change */
void SetView()
{
    /*glm::mat4 Trans = glm::translate(glm::mat4(1.0f), glm::vec3(-1, 0, g_trans));
    safe_glUniformMatrix4fv(h_uViewMatrix, glm::value_ptr(Trans));*/

    glm::mat4 Trans = glm::translate(glm::mat4(1.0f), glm::vec3(0.0f, 0, g_trans));
    glm::mat4 RotateX = glm::rotate(Trans, g_angle, glm::vec3(0.0f, 1, 0));
    safe_glUniformMatrix4fv(h_uViewMatrix, glm::value_ptr(RotateX));
}

void SetModelI()
{
    glm::mat4 Ident(1.0f);
    safe_glUniformMatrix4fv(h_uModelMatrix, glm::value_ptr(Ident));
}

/* model transforms */
void SetModel(RenderingHelper modelTrans)
{
    safe_glUniformMatrix4fv(h_uModelMatrix, glm::value_ptr(modelTrans.modelViewMatrix));
    safe_glUniformMatrix4fv(h_uTpModelMatrix, glm::value_ptr(
                                                             glm::transpose(glm::inverse(modelTrans.modelViewMatrix))));
}

void setColors(int mode, vec3 ambColor)
{
    switch (mode)
    {
        //text
    case 0:
        glUniform3f(h_uAmbColor, 0, 0, 0);
        glUniform3f(h_uDiffColor, 0, 0, 0);
        glUniform3f(h_uSpecColor, 0, 0, 0);
        glUniform1f(h_uShine, 0);
        break;
        //bird
    case 1:
        glUniform3f(h_uAmbColor, ambColor.x, ambColor.y, ambColor.z);
        glUniform3f(h_uDiffColor, .3, .3, .3);
        glUniform3f(h_uSpecColor, .3, .3, .3);
        glUniform1f(h_uShine, 100.0);
        break;
        //bird eyes
    case 2:
        glUniform3f(h_uAmbColor, 0, 0, 0);
        glUniform3f(h_uDiffColor, .3, .3, .3);
        glUniform3f(h_uSpecColor, .6, .6, .6);
        glUniform1f(h_uShine, 5.0);
        break;
        //bird beak
    case 3:
        glUniform3f(h_uAmbColor, .7, .7, 0.0);
        glUniform3f(h_uDiffColor, .3, .3, .3);
        glUniform3f(h_uSpecColor, .3, .3, .3);
        glUniform1f(h_uShine, 5.0);
        break;
        //would be bird's shadow
    case 4:
        glUniform3f(h_uAmbColor, 0, 0, 0);
        glUniform3f(h_uDiffColor, .3, .3, .3);
        glUniform3f(h_uSpecColor, .6, .6, .6);
        glUniform1f(h_uShine, 5.0);
        break;
        //baseball
    case 5:
        glUniform3f(h_uAmbColor, .8, .8, .8);
        glUniform3f(h_uDiffColor, .3, .3, .3);
        glUniform3f(h_uSpecColor, .3, .3, .3);
        glUniform1f(h_uShine, 100.0);
    }
}

/*function to help load the shaders (both vertex and fragment */
int InstallShader(const GLchar *vShaderName, const GLchar *fShaderName, int &ShadeProg)
{
    GLuint VS; //handles to shader object
    GLuint FS; //handles to frag shader object
    GLint vCompiled, fCompiled, linked; //status of shader

    VS = glCreateShader(GL_VERTEX_SHADER);
    FS = glCreateShader(GL_FRAGMENT_SHADER);

    //load the source
    glShaderSource(VS, 1, &vShaderName, NULL);
    glShaderSource(FS, 1, &fShaderName, NULL);

    //compile shader and print log
    glCompileShader(VS);
    /* check shader status requires helper functions */
    printOpenGLError();
    glGetShaderiv(VS, GL_COMPILE_STATUS, &vCompiled);
    printShaderInfoLog(VS);

    //compile shader and print log
    glCompileShader(FS);
    /* check shader status requires helper functions */
    printOpenGLError();
    glGetShaderiv(FS, GL_COMPILE_STATUS, &fCompiled);
    printShaderInfoLog(FS);

    if (!vCompiled || !fCompiled)
    {
        printf("Error compiling either shader %s or %s", vShaderName, fShaderName);
        return 0;
    }

    //create a program object and attach the compiled shader
    ShadeProg = glCreateProgram();
    glAttachShader(ShadeProg, VS);
    glAttachShader(ShadeProg, FS);

    glLinkProgram(ShadeProg);
    /* check shader status requires helper functions */
    printOpenGLError();
    glGetProgramiv(ShadeProg, GL_LINK_STATUS, &linked);
    printProgramInfoLog(ShadeProg);

    glUseProgram(ShadeProg);

    printf("sucessfully installed shader %d\n", ShadeProg);
    return 1;
}

/* Some OpenGL initialization */
void Initialize() // Any GL Init Code
{
    // Start Of User Initialization
    glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
    glClearColor(172.0 / 255.0, 216.0 / 255.0, 230.0 / 255.0, 1.0f);
    // Black Background
    glClearDepth(1.0f); // Depth Buffer Setup
    glDepthFunc(GL_LEQUAL); // The Type Of Depth Testing
    glEnable(GL_DEPTH_TEST); // Enable Depth Testing


    LoadTexture((char*) "textures/baseball_color.bmp", 0);
    LoadTexture((char*) "textures/whiteTexture.bmp", 1);
    g_textureid = 2;
    LoadTexture((char*) "textures/clouds.bmp", g_textureid);
    LoadTexture((char*) "textures/cloudTexture.bmp", 3);
}

void setLightSource()
{
    GLfloat DiffuseLight[] = {1, 1, 1};
    GLfloat AmbientLight[] = {1, 1, 1};

    glLightfv(GL_LIGHT0, GL_DIFFUSE, DiffuseLight);
    glLightfv(GL_LIGHT0, GL_AMBIENT, AmbientLight);

    GLfloat LightPosition[] = {0, 0, 50, 0};
    glLightfv(GL_LIGHT0, GL_POSITION, LightPosition);

    glUniform4f(h_uLColor, 1.0, 1.0, 1.0, 1.0);
}

float p2wx(int pixel_x)
{
    float width = g_width;

    float world_x = (2.0 / (width - 1.0)) * pixel_x - 1.0;

    return MOUSE_SCALE * world_x;
}

float p2wy(int pixel_y)
{
    float height = g_height;

    //flip glut y
    pixel_y = height - pixel_y;

    float world_y = (2.0 / (height - 1.0)) * pixel_y - 1.0;

    return MOUSE_SCALE * world_y;
}

int w2px(float world_x)
{
    float width = g_width;

    world_x /= MOUSE_SCALE;

    float pixel_x = (1.0 + world_x) * (width - 1.0) / 2.0;\

    return (int) round(pixel_x);
}

int w2py(float world_y)
{
    float height = g_height;

    world_y /= MOUSE_SCALE;

    float pixel_y = (1.0 + world_y) * (height - 1.0) / 2.0;

    //flip glut y
    pixel_y = height - pixel_y;

    return (int) round(pixel_y);
}

void advanceState(State* newState)
{
    delete currState;
    currState = newState;
    glutPostRedisplay();
}

/* Main display function */
void Draw(void)
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    //Start our shader
    glUseProgram(ShadeProg);
    SetProjectionMatrix();
    SetView();

    setLightSource();

    //Draw Background clouds and stuff
    RenderingHelper ModelTrans;
    ModelTrans.useModelViewMatrix();
    ModelTrans.loadIdentity();

    GLUquadricObj *mySphere = gluNewQuadric();
    gluQuadricDrawStyle(mySphere, GLU_FILL);
    gluQuadricNormals(mySphere, GLU_SMOOTH);
    gluQuadricTexture(mySphere, GL_TRUE);

    glEnable(GL_TEXTURE_2D);
    glActiveTexture(GL_TEXTURE0);
    glBindTexture(GL_TEXTURE_2D, g_textureid);
    int texLocation = glGetUniformLocation(getShade(), "uTexLocation");
    safe_glUniform1i(texLocation, 0);

    setColors(5, vec3(0));
    ModelTrans.pushMatrix();
    ModelTrans.translate(vec3(0, 0, -90));
    SetModel(ModelTrans);
    gluDisk(mySphere, 0, 200, 12, 12);
    ModelTrans.popMatrix();
    glDisable(GL_TEXTURE_2D);

    // draw!
    currState->draw();

    //Disable the shader
    glUseProgram(0);
    glutSwapBuffers();
}

/* Reshape */
void ReshapeGL(int width, int height)
{
    g_width = (float) width;
    g_height = (float) height;
    glViewport(0, 0, (GLsizei) (width), (GLsizei) (height));
}

static void timer(int value)
{
    // update models
    currState->update();

    // redraw view
    glutPostRedisplay();

    // continue controller loop
    glutTimerFunc(TIMER, timer, 0);
}

//the keyboard callback

void Keyboard(unsigned char key, int x, int y)
{
    switch (key)
    {
    case 'p': case 'P':
        if (currState->isPaused())
        {
            currState->resume();
        }
        else
        {
            currState->pause();
        }
        break;
    case 'q': case 'Q':
        exit(EXIT_SUCCESS);
        break;
    }
    glutPostRedisplay();
}

void MouseMotion(int x, int y)
{
    currState->mouseMoved(x, y);
}

void Mouse(int button, int state, int x, int y)
{
    currState->mouseClicked(button, state, x, y);
}

int main(int argc, char *argv[])
{
    currState = new MainMenu();

    g_width = 700;
    g_height = 700;

    glutInit(&argc, argv);
    glutInitWindowPosition(20, 20);
    glutInitWindowSize(g_width, g_height);
    glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
    glutCreateWindow("Home Run");

    glutReshapeFunc(ReshapeGL);
    glutDisplayFunc(Draw);
    glutKeyboardFunc(Keyboard);
    glutMouseFunc(Mouse);
    glutPassiveMotionFunc(MouseMotion);
    glutTimerFunc(TIMER, timer, 0);

    Initialize();

    //test the openGL version
    getGLversion();
    //install the shader
    if (!InstallShader(textFileRead((char *) "FinalProject_vert.glsl"),
                       textFileRead((char *) "FinalProject_frag.glsl"), ShadeProg))
    {
        printf("Error installing shader!\n");
        return 0;
    }

    /* get handles to attribute data */
    h_aPosition = safe_glGetAttribLocation(ShadeProg, "aPosition");
    h_uAmbColor = safe_glGetUniformLocation(ShadeProg, "uAmbColor");
    h_uDiffColor = safe_glGetUniformLocation(ShadeProg, "uDiffColor");
    h_uSpecColor = safe_glGetUniformLocation(ShadeProg, "uSpecColor");
    h_uShine = safe_glGetUniformLocation(ShadeProg, "uShine");

    h_uLColor = safe_glGetUniformLocation(ShadeProg, "uLColor");

    h_uProjMatrix = safe_glGetUniformLocation(ShadeProg, "uProjMatrix");
    h_uViewMatrix = safe_glGetUniformLocation(ShadeProg, "uViewMatrix");
    h_uModelMatrix = safe_glGetUniformLocation(ShadeProg, "uModelMatrix");
    h_uTpModelMatrix = safe_glGetUniformLocation(ShadeProg, "uTpModelMatrix");


    if (!InstallShader(textFileRead((char *) "objects/Cloud_vert.glsl"),
                       textFileRead((char *) "objects/Cloud_frag.glsl"), CloudShadeProg))
    {
        printf("Error installing cloud shader!\n");
        return 0;
    }
    //glutFullScreen();

    glutMainLoop();

    delete currState;

    return 0;
}
