/* 
 * File:   bird.cpp
 * Author: goofanader
 * 
 * Created on May 23, 2012, 4:31 PM
 */

#include "Bird.h"
#include "../FinalProject.h"

#define COLLISION_RADIUS 1
#define WING_SPEED 30
#define MAX_FLAP_ANGLE 70

Bird::Bird(glm::vec3 newPosition, glm::vec3 newVelocity, glm::vec3 newAcceleration)
{
    wingFlapAngle = 0;
    wingFlapDirection = 1;
    
    velocity = newVelocity;
    position = newPosition;
    acceleration = newAcceleration;

    //make a fake bird color!
    color = glm::vec3(RAND_COLOR, RAND_COLOR, RAND_COLOR);
    //printf("color: %f, %f, %f\n", color.x, color.y, color.z);
    scaleFactor = glm::vec3(RAND_SCALE);

    textureid = 1;
    
}

Bird::Bird(const Bird& orig)
{
}

Bird::~Bird()
{
}

void Bird::drawEnemy()
{
    vec3 radiusVec = vec3(0);

    RenderingHelper ModelTrans;
    ModelTrans.useModelViewMatrix();
    ModelTrans.loadIdentity();
    
    GLUquadricObj *mySphere = gluNewQuadric();
    gluQuadricDrawStyle(mySphere, GLU_FILL);
    gluQuadricNormals(mySphere, GL_TRUE);
    gluQuadricTexture(mySphere, GL_TRUE);

    glEnable(GL_TEXTURE_2D);
    glActiveTexture(GL_TEXTURE0);
    glBindTexture(GL_TEXTURE_2D, textureid);
    int texLocation = glGetUniformLocation(getShade(), "uTexLocation");
    safe_glUniform1i(texLocation, 0);

    //printf("cawcaw\n");
    setColors(1, color);
    ModelTrans.pushMatrix();
        ModelTrans.translate(position);

        ModelTrans.rotate((180.0 / 3.141592653) * glm::atan(velocity.y, velocity.x) - 90, vec3(0, 0, 1));
        ModelTrans.rotate(-90, vec3(1,0,0));

        ModelTrans.scale(scaleFactor.x, scaleFactor.y, scaleFactor.z);

        //draw head
        ModelTrans.pushMatrix();
            ModelTrans.translate(vec3(0,.5,1.5));
            //draw beak
            setColors(3, color);
            ModelTrans.pushMatrix();
                ModelTrans.translate(vec3(0,0,.5));
                ModelTrans.scale(.4, .4, .5);
                SetModel(ModelTrans);
                glutSolidCone(1,1,12,12);
            ModelTrans.popMatrix();

            //draw left eye
            setColors(2, color);
            ModelTrans.pushMatrix();
                ModelTrans.translate(vec3(.5,.2,.25));
                ModelTrans.scale(.2);
                SetModel(ModelTrans);
                gluSphere(mySphere, 1, 12, 12);
            ModelTrans.popMatrix();

            //draw right eye
            ModelTrans.pushMatrix();
                ModelTrans.translate(vec3(-.5,.2,.25));
                ModelTrans.scale(.2);
                SetModel(ModelTrans);
                gluSphere(mySphere, 1, 12, 12);
            ModelTrans.popMatrix();

            //continue drawing head
            setColors(1, color);
            SetModel(ModelTrans);
            gluSphere(mySphere, .7, 12, 12);
        ModelTrans.popMatrix();

        //draw body
        ModelTrans.pushMatrix();
            ModelTrans.translate(vec3(0,-.5,0));
            radiusVec += vec3(0,.5,0) * vec3(1,1,1.5);
            ModelTrans.scale(1,1,1.5);
            SetModel(ModelTrans);
            gluSphere(mySphere, 1, 12, 12);
        ModelTrans.popMatrix();

        //draw left wing
        ModelTrans.pushMatrix();
            ModelTrans.translate(vec3(.5,-.25,.25));
            ModelTrans.rotate(90,vec3(0,1,0));
            ModelTrans.rotate(wingFlapAngle, vec3(1,0,0));
            ModelTrans.scale(1,.5,2);
            SetModel(ModelTrans);
            glutSolidCone(1,1,12,12);
        ModelTrans.popMatrix();

        //draw right wing
        ModelTrans.pushMatrix();
            ModelTrans.translate(vec3(-.5,-.25,.25));
            ModelTrans.rotate(-90,vec3(0,1,0));
            ModelTrans.rotate(wingFlapAngle, vec3(1,0,0));
            ModelTrans.scale(1,.5,2);
            SetModel(ModelTrans);
            glutSolidCone(1,1,12,12);
        ModelTrans.popMatrix();

        //draw tail
        ModelTrans.pushMatrix();
            ModelTrans.translate(vec3(0,-.25,-2));
            ModelTrans.scale(1,.5,1);
            SetModel(ModelTrans);
            glutSolidCone(1,1,12,12);
        ModelTrans.popMatrix();

        //draw bounding sphere
        //ModelTrans.pushMatrix();
          //  SetModel(ModelTrans);
            //gluSphere(mySphere,COLLISION_RADIUS,12,12);
        //ModelTrans.popMatrix();
        
    ModelTrans.popMatrix();
    glDisable(GL_TEXTURE_2D);

    gluDeleteQuadric(mySphere);
}

void Bird::updateEnemy()
{
    Enemy::updateEnemy();
    wingFlapAngle += WING_SPEED * wingFlapDirection;

    if (wingFlapAngle > MAX_FLAP_ANGLE || wingFlapAngle < -MAX_FLAP_ANGLE)
        wingFlapDirection *= -1;
}

double Bird::getRadius()
{
    return COLLISION_RADIUS;
}