#include "Cloud.h"
#include "Plane.h"

#define RAND_NUM_SPHERES ((rand() % (MAX_SPHERES - MIN_SPHERES)) + MIN_SPHERES)
#define RAND_SPHERE_POS ((((float)rand()) / RAND_MAX * 5) - .2)
#define RAND_SPHERE_SCALE ((((float)rand()) / RAND_MAX * 2) + .5)

Cloud::Cloud(glm::vec3 newPosition, glm::vec3 newVelocity, glm::vec3 newAcceleration)
{
    position = newPosition;
    velocity = newVelocity;
    acceleration = newAcceleration;
    
    numSpheres = RAND_NUM_SPHERES;
    textureid = 3;

    for (int i = 0; i < numSpheres; i++)
    {
        spherePos[i] = vec3(RAND_SPHERE_POS, RAND_SPHERE_POS, RAND_SPHERE_POS);
        sphereScale[i] = vec3(RAND_SPHERE_SCALE,RAND_SPHERE_SCALE,RAND_SPHERE_SCALE);
        sphereAngle[i] = glm::degrees((float)rand() / RAND_MAX - .5);

        int randAxis = rand() % 3;
        sphereType[i] = randAxis;

        if (randAxis == 0) {
            sphereAngleAxis[i] = vec3(1,0,0);
        }
        else if (randAxis == 1) {
            sphereAngleAxis[i] = vec3(0,1,0);
        }
        else {
            sphereAngleAxis[i] = vec3(0,0,1);
        }
    }
}

Cloud::Cloud(const Cloud& orig)
{
}

Cloud::~Cloud()
{
}

void Cloud::drawEnemy()
{
    //glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    //Disable the shader
    //glUseProgram(0);
    //glutSwapBuffers();
    //Start our shader
    glUseProgram(getCloudShade());
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

    //set projection matrix
    glm::mat4 Projection = glm::perspective(80.0f, (float) getWidth() / getHeight(), 0.1f, 100.f);
    safe_glUniformMatrix4fv(glGetUniformLocation(getCloudShade(), "uProjMatrix"),
                            glm::value_ptr(Projection));

    //set view
    glm::mat4 Trans = glm::translate(glm::mat4(1.0f), glm::vec3(0.0f, 0, getTrans()));
    glm::mat4 RotateX = glm::rotate(Trans, getAngle(), glm::vec3(0.0f, 1, 0));
    safe_glUniformMatrix4fv(glGetUniformLocation(getCloudShade(), "uViewMatrix"),
                            glm::value_ptr(RotateX));

    GLUquadricObj *mySphere = gluNewQuadric();
    gluQuadricDrawStyle(mySphere, GLU_FILL);
    gluQuadricNormals(mySphere, GL_SMOOTH);
    gluQuadricTexture(mySphere, GL_TRUE);

    RenderingHelper* ModelTrans = new RenderingHelper();

    glEnable(GL_TEXTURE_2D);
    glActiveTexture(GL_TEXTURE0);
    glBindTexture(GL_TEXTURE_2D, textureid);
    int texLocation = glGetUniformLocation(getCloudShade(), "uTexLocation");
    safe_glUniform1i(texLocation, 0);

    ModelTrans->useModelViewMatrix();
        ModelTrans->loadIdentity();
        
    ModelTrans->pushMatrix();
    ModelTrans->translate(position);

    for (int i = 0; i < numSpheres; i++)
    {
        ModelTrans->pushMatrix();
            ModelTrans->translate(spherePos[i]);
            ModelTrans->rotate(sphereAngle[i], sphereAngleAxis[i]);
            ModelTrans->scale(sphereScale[i].x, sphereScale[i].y, sphereScale[i].z);

            //set model transforms
            safe_glUniformMatrix4fv(glGetUniformLocation(getCloudShade(), "uModelMatrix"),
                                    glm::value_ptr(ModelTrans->modelViewMatrix));

            if (sphereType[i] == 0)
                gluSphere(mySphere, 1, 12, 12);
            else if (sphereType[i] == 1)
                //gluCylinder(mySphere, 1, .5, 1, 12,12);
                //gluSphere(mySphere, 1, 12, 12);
                gluDisk(mySphere, 0, 1, 12, 12);
            else
                gluDisk(mySphere, 0, 1, 12, 12);
        
        ModelTrans->popMatrix();
    }

    ModelTrans->popMatrix();
    glDisable(GL_TEXTURE_2D);

    gluDeleteQuadric(mySphere);
    //Disable the shader
    //glUseProgram(0);
    //glutSwapBuffers();

    delete ModelTrans;

    glDisable(GL_BLEND);
    glUseProgram(getShade());
}

double Cloud::getRadius()
{
    return 0;
}

bool Cloud::isCloud()
{
    return true;
}